@@ -1,6 +1,6 @@
 # -*-python-*-
 #
-# Copyright (C) 1999-2008 The ViewCVS Group. All Rights Reserved.
+# Copyright (C) 1999-2012 The ViewCVS Group. All Rights Reserved.
 #
 # By using this file, you agree to the terms and conditions set forth in
 # the LICENSE.html file which can be found at the top level of the ViewVC
@@ -14,6 +14,7 @@ import os.path
 
 
 def canonicalize_rootpath(rootpath):
+  assert os.path.isabs(rootpath)
   return os.path.normpath(rootpath)
 
 
@@ -22,6 +23,7 @@ def expand_root_parent(parent_path):
   # "CVSROOT/config" is added the set of returned roots.  Or, if the
   # PARENT_PATH itself contains a child "CVSROOT/config", then all its
   # subdirectories are returned as roots.
+  assert os.path.isabs(parent_path)
   roots = {}
   subpaths = os.listdir(parent_path)
   cvsroot = os.path.exists(os.path.join(parent_path, "CVSROOT", "config"))
